"""This module contains the standard time format for the application."""

import datetime

DEFAULT_TIME_FORMAT = "%Y-%m-%dT%H:%M:%S%z"
"""Default time format and time zone %Y-%m-%dT%H:%M:%S%z"""
DEFAULT_TIME_ZONE = datetime.UTC
"""Default time zone is UTC(+00:00)"""



"""InMemorySessionStore is a simple in-memory session store for the authentication provider."""

import logging
from datetime import datetime
from typing import Any

from pydantic import BaseModel

logger = logging.getLogger(__name__)


class SessionData(BaseModel):
    """SessionData is a model for the session data."""

    user_info: dict[str, Any]
    access_token: str
    refresh_token: str
    id_token: str
    expires_at: datetime | None = None



"""Routes module for the authentication provider."""

from fastapi import Request
from fastapi.exceptions import HTTPException

from zmp_authentication_provider.service.auth_service import AuthService
from zmp_authentication_provider.setting import auth_default_settings
from zmp_authentication_provider.utils.redis_session_store import RedisSessionStore


async def get_auth_service(request: Request) -> AuthService:
    """Get the auth service."""
    service = getattr(request.app.state, auth_default_settings.service_name, None)
    if not service:
        raise HTTPException(
            status_code=500,
            detail=f"Service '{auth_default_settings.service_name}' not available in the request state. "
            "You should set the service in the request state.",
        )

    return service


async def get_redis_session_store(request: Request) -> RedisSessionStore:
    """Get the redis session store."""
    redis_session_store = getattr(request.app.state, "redis_session_store", None)
    if not redis_session_store:
        raise HTTPException(
            status_code=500,
            detail="Redis session store not available in the request state. "
            "You should set the redis session store in the request state.",
        )
    return redis_session_store


__all__ = ["get_auth_service", "get_redis_session_store"]



"""InMemorySessionStore is a simple in-memory session store for the authentication provider."""

import logging
import time
from datetime import datetime, timedelta
from threading import Lock, Thread

from zmp_authentication_provider.utils.session_data import SessionData
from zmp_authentication_provider.utils.time_utils import DEFAULT_TIME_ZONE

logger = logging.getLogger(__name__)


class InMemorySessionStore:
    """InMemorySessionStore is a simple in-memory session store for the authentication provider."""

    def __init__(self, session_ttl: int = 3600, session_cleanup_interval: int = 300):
        """Initialize the InMemorySessionStore."""
        self.session_store: dict[str, SessionData] = {}
        self.session_ttl = session_ttl
        self.session_cleanup_interval = session_cleanup_interval
        self.session_store_lock = Lock()
        self._running = True
        self.session_cleanup_thread = Thread(target=self.session_cleanup, daemon=True)
        self.session_cleanup_thread.start()

    def get(self, unique_id: str) -> SessionData:
        """Get the session data from the in-memory session store."""
        return self.session_store.get(unique_id)

    def set(self, unique_id: str, session_data: SessionData):
        """Set the session data in the in-memory session store."""
        # delete the session data if it exists
        if self.get(unique_id):
            self.delete(unique_id)

        # set the expires_at
        session_data.expires_at = datetime.now(DEFAULT_TIME_ZONE) + timedelta(
            seconds=self.session_ttl
        )

        # set the session data in the in-memory session store
        self.session_store[unique_id] = session_data

    def reset_ttl(self, unique_id: str):
        """Reset the ttl of the session data in the in-memory session store."""
        session_data = self.get(unique_id)
        if session_data:
            session_data.expires_at = datetime.now(DEFAULT_TIME_ZONE) + timedelta(
                seconds=self.session_ttl
            )
            self.session_store[unique_id] = session_data
            logger.info(
                f"Session data for {unique_id} has been reset ttl until {session_data.expires_at}"
            )

    def delete(self, unique_id: str):
        """Delete the session data from the in-memory session store."""
        if self.get(unique_id):
            del self.session_store[unique_id]
        else:
            # skip if the session data is not found in the in-memory session store
            pass

    def session_cleanup(self):
        """Cleanup the session data from the in-memory session store."""
        while self._running:
            with self.session_store_lock:
                for unique_id, session_data in self.session_store.items():
                    if (
                        session_data.expires_at
                        and session_data.expires_at < datetime.now(DEFAULT_TIME_ZONE)
                    ):
                        self.delete(unique_id)
                        logger.info(
                            f"Session data for {unique_id} has expired and has been deleted."
                        )

                logger.info(
                    f"Session cleanup completed. {len(self.session_store)} sessions remaining."
                )

            time.sleep(self.session_cleanup_interval)

    def close(self):
        """Explicitly clean up resources."""
        self._running = False
        if self.session_cleanup_thread.is_alive():
            self.session_cleanup_thread.join(timeout=5)



"""OAuth2 Keycloak module for the AIops Pilot."""

import logging
from typing import Any

from fastapi import HTTPException, Request, status

from zmp_authentication_provider.setting import auth_default_settings

log = logging.getLogger(__name__)

USER_SESSION_KEY = "user_info"


async def get_session_id_in_cookie(request: Request) -> str:
    """Get the session id from the cookie."""
    session_id: str = request.cookies.get(auth_default_settings.session_id_cookie_name)
    if not session_id:
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Session id is not found in the request session",
        )

    return session_id



"""Base settings for MCP and Gateway."""

from pydantic_settings import BaseSettings, SettingsConfigDict


class KeycloakSettings(BaseSettings):
    """Settings for Keycloak."""

    model_config: SettingsConfigDict = SettingsConfigDict(
        env_prefix="KEYCLOAK_", env_file=".env", extra="ignore"
    )

    server_url: str
    realm: str
    client_id: str
    client_secret: str
    redirect_uri: str
    algorithm: str = "RS256"
    scope: str = "openid profile email"


keycloak_settings = KeycloakSettings()


class RedisSettings(BaseSettings):
    """Settings for Redis."""

    model_config: SettingsConfigDict = SettingsConfigDict(
        env_prefix="REDIS_", env_file=".env", extra="ignore"
    )

    host: str = "localhost"
    port: int = 6379
    db: int = 0
    password: str | None = None
    decode_responses: bool = True
    session_prefix: str = "session:"
    session_ttl: int = 3600  # 1 hour default


redis_settings = RedisSettings()


class AuthDefaultSettings(BaseSettings):
    """Settings for Basic Auth."""

    model_config: SettingsConfigDict = SettingsConfigDict(
        env_prefix="AUTH_", env_file=".env", extra="ignore"
    )

    # The name of the auth service
    service_name: str = "auth_service"

    # The api endpoint of the application which is used to redirect to the login page
    application_endpoint: str

    # The secret key for the encryption
    basic_auth_encryption_key: str = (
        "425df05bf30c8434e8a619563b602a7aa0421011f71727289eee66d310897118"
    )

    # The collection name for the basic auth user
    basic_auth_user_collection: str = "basic_auth_user"

    # The collection name for the OAuth user
    oauth_user_collection: str = "oauth_user"

    # The http client ssl verify
    http_client_ssl_verify: bool = True

    # The http client timeout
    http_client_timeout: int = 30

    # The csrf token cookie name
    csrf_token_cookie_name: str = "csrftoken"

    # The csrf token header name
    csrf_token_header_name: str = "x-csrftoken"

    # The session id cookie name
    session_id_cookie_name: str = "session_id"

    # state separator
    state_separator: str = "::"

    # Session max_age (cookie expiration time)
    session_max_age: int = 3600  # 1 hour

    # Session middleware same_site policy
    session_same_site: str = "lax"

    # Session middleware https_only
    session_https_only: bool = False

    # Session cookie secure
    session_secure: bool = False

    # Session middleware domain
    session_domain: str = "0.0.0.0"

    # Session memory ttl
    session_ttl: int = 3600  # 1 hour default
    session_cleanup_interval: int = 300  # 5 minutes default

    # Flag to enable the OAuth user creation
    enable_oauth_user_creation: bool = False


auth_default_settings = AuthDefaultSettings()
